@@ -10,6 +10,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+
 import re
 import os
 import sys
@@ -19,6 +20,7 @@ import socket
 import hashlib
 import textwrap
 
+import traceback
 import logging
 
 # logging.basicConfig(level=logging.INFO)
@@ -28,6 +30,13 @@ import requests
 import six
 from six.moves.urllib.parse import quote, urlparse
 from markdown2 import markdown as _markdown
+import yaml
+
+
+def u(text):
+    if sys.version_info[0] < 3:
+        return unicode(text).encode("utf-8")
+    return text
 
 
 def markdown(*args, **kwargs):
@@ -50,7 +59,7 @@ def markdown(*args, **kwargs):
 
 AUTHOR = "Xvezda"
 AUTHOR_EMAIL = "xvezda@naver.com"
-VERSION = "1.0.0"
+VERSION = "1.0.2"
 API_VERSION = "v1"
 BASE_URL = "https://www.tistory.com"
 PORT = int(os.getenv("TISPOON_PORT", 9638))
@@ -69,7 +78,12 @@ COMMENT_SECRET = 1
 COMMENT_PUBLIC = 0
 
 
-DEMO_MARKDOWN = r"""
+DEMO_MARKDOWN = """\
+---
+title: Tispoon 테스트
+visibility: 1
+---
+
 # Hello World!
 
 파이썬 기반 티스토리 블로깅 라이브러리
@@ -303,7 +317,7 @@ class Tispoon(TispoonBase):
         for keyname in kwargs:
             if kwargs[keyname] is None:
                 continue
-            ret += "&%s=%s" % (keyname, quote(str(kwargs[keyname])))
+            ret += "&%s=%s" % (keyname, quote(u(kwargs[keyname])))
         return ret
 
     def blog_info(self):
@@ -312,9 +326,11 @@ class Tispoon(TispoonBase):
             r = self.cache.get(url, requests.get)
         else:
             r = requests.get(url)
-        res = json.loads(r.text)
+        res = json.loads(r.text, encoding="utf-8")
         if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
         return res.get("tistory").get("item").get("blogs")
 
     def default_blog(self):
@@ -332,9 +348,11 @@ class Tispoon(TispoonBase):
             r = self.cache.get(url, requests.get)
         else:
             r = requests.get(url)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
         return res
 
     def post_list(self, page=1):
@@ -364,9 +382,11 @@ class Tispoon(TispoonBase):
         url = self.assemble_url("post/read", blogName=self.blog, postId=post_id)
 
         r = requests.get(url)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
         return dotget(res, "tistory.item")
 
     def post_write(self, post):
@@ -383,13 +403,12 @@ class Tispoon(TispoonBase):
             acceptComment=post.get("accept_comment"),
             password=post.get("password"),
         )
-        if self.cache:
-            r = self.cache.get(url, requests.get)
-        else:
-            r = requests.get(url)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
+        r = requests.post(url)
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
 
         return {
             "post_id": dotget(res, "tistory.postId"),
@@ -412,9 +431,11 @@ class Tispoon(TispoonBase):
             password=post.get("password"),
         )
         r = requests.post(url)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
 
         return {
             "post_id": dotget(res, "tistory.postId"),
@@ -429,9 +450,11 @@ class Tispoon(TispoonBase):
         url = self.assemble_url("post/attach", blogName=self.blog)
 
         r = requests.post(url, files=files)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
 
         return {
             "url": dotget(res, "tistory.url"),
@@ -439,13 +462,24 @@ class Tispoon(TispoonBase):
         }
 
     def post_demo(self):
-        return self.post_write(
-            {
-                "title": "Tispoon 테스트",
-                "content": markdown(DEMO_MARKDOWN),
-                "visibility": VISIBILITY_PUBLISHED,
-            }
-        )
+        post = self.markdown_to_post(DEMO_MARKDOWN)
+        return self.post_write(post)
+        # return self.post_write(
+        #     {
+        #         "title": "Tispoon 테스트",
+        #         "content": markdown(DEMO_MARKDOWN),
+        #         "visibility": VISIBILITY_PUBLISHED,
+        #     }
+        # )
+
+    def markdown_to_post(self, md):
+        metadata = re.match("""^---\s(.+?)\s---""", md, flags=re.S)
+        if metadata:
+            post = yaml.load(metadata.group(1), Loader=yaml.BaseLoader)
+            content = re.sub("""^---\s(.+?)\s---\s*""", "", md, flags=re.S)
+            post["content"] = markdown(content)
+            return post
+        return {"content": markdown(md)}
 
     def category_list(self):
         url = self.assemble_url("category/list", blogName=self.blog)
@@ -454,9 +488,11 @@ class Tispoon(TispoonBase):
             r = self.cache.get(url, requests.get)
         else:
             r = requests.get(url)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
 
         return dotget(res, "tistory.item.categories")
 
@@ -468,18 +504,22 @@ class Tispoon(TispoonBase):
             r = self.cache.get(url, requests.get)
         else:
             r = requests.get(url)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
 
         return dotget(res, "tistory.item.comments.comment")
 
     def comment_list(self, post_id):
         url = self.assemble_url("comment/list", blogName=self.blog, postId=post_id)
         r = requests.get(url)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
 
         return dotget(res, "tistory.item.comments.comment")
 
@@ -493,9 +533,11 @@ class Tispoon(TispoonBase):
             secret=comment.get("secret"),
         )
         r = requests.post(url)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
 
         return dotget(res, "tistory.commentUrl")
 
@@ -510,9 +552,11 @@ class Tispoon(TispoonBase):
             secret=comment.get("secret"),
         )
         r = requests.post(url)
-        if r.status_code != 200:
-            raise TispoonError(res.get("error_message") or "unexpected error")
         res = json.loads(r.text)
+        if r.status_code != 200:
+            raise TispoonError(
+                dotget(res, "tistory.error_message") or "unexpected error"
+            )
 
         return dotget(res, "tistory.commentUrl")
 
@@ -524,36 +568,70 @@ class Tispoon(TispoonBase):
             commentId=comment.get("comment_id"),
         )
         r = requests.post(url)
+        res = json.loads(r.text)
         if r.status_code != 200:
             # raise TispoonError(res.get('error_message') or 'unexpected error')
-            logger.error(res.get("error_message") or "unexpected error")
+            logger.error(dotget(res, "tistory.error_message") or "unexpected error")
             return False
-        res = json.loads(r.text)
 
         return True
 
 
-if __name__ == "__main__":
+def main():
+    try:
+        import dotenv
+
+        dotenv.load_dotenv()
+    except ImportError:
+        pass
+
     import argparse
 
     parser = argparse.ArgumentParser()
-    parser.add_argument("--token")
-    parser.add_argument("--client-id")
-    parser.add_argument("--client-secret")
+    parser.add_argument("--token", "-t")
+    parser.add_argument("--client-id", "-u")
+    parser.add_argument("--client-secret", "-p")
     parser.add_argument(
-        "--blog", help="specify blog name. (i.e. [blogName].tistory.com)"
+        "--list", "-l", action="store_true", help="list blog informations"
+    )
+    parser.add_argument("--file", "-f", action="append", help="markdown file to post")
+    parser.add_argument(
+        "--blog", "-b", help="specify blog name. (i.e. [blogName].tistory.com)"
     )
     parser.add_argument(
-        "--demo", action="store_true", help="posting demo article to blog."
+        "--demo", "-d", action="store_true", help="posting demo article to blog."
     )
-    parser.add_argument("--version", action="version", version=VERSION)
+    parser.add_argument("--verbose", "-v", action="count", default=0)
+    parser.add_argument("--version", "-V", action="version", version=VERSION)
     args = parser.parse_args()
 
     try:
         t = Tispoon(token=args.token, blog=args.blog)
         if args.demo:
             t.post_demo()
+        elif args.file:
+            for mdfile in args.file:
+                print("Posting %s..." % mdfile)
+                with open(mdfile, "r") as f:
+                    t.post_write(t.markdown_to_post(f.read()))
         else:
-            print(json.dumps(t.blogs, indent=4))
-    except Exception:
+            for blog in t.blogs:
+                print(
+                    textwrap.dedent(
+                        """\
+                    - name: %s
+                      title: %s
+                      url: %s
+                """
+                        % (blog.get("name"), blog.get("title"), blog.get("url"))
+                    )
+                )
+    except Exception as err:
+        if args.verbose > 0:
+            print(traceback.format_exc(), file=sys.stderr)
+        parser.error(u(err))
         parser.print_help()
+
+
+if __name__ == "__main__":
+    main()
